@@ -1,7 +1,7 @@
-/* $Id: lukemftp.h,v 1.13 1999/11/13 01:40:40 lukem Exp $ */
+/* $Id: lukemftp.h,v 1.20 2000/02/03 09:20:51 lukem Exp $ */
 
 #define	FTP_PRODUCT	"lukemftp"
-#define	FTP_VERSION	"1.1"
+#define	FTP_VERSION	"1.2"
 
 #include "config.h"
 
@@ -106,9 +106,6 @@
 # define _PATH_TMP	"/tmp/"
 #endif
 
-#if HAVE_STRINGLIST_H
-# include <stringlist.h>
-#else	/* ! HAVE_STRINGLIST_H */
 typedef struct _stringlist {
 	char	**sl_str;
 	size_t	  sl_max;
@@ -116,10 +113,9 @@ typedef struct _stringlist {
 } StringList;
 
 StringList *sl_init	__P((void));
-void	 sl_add		__P((StringList *, char *));
+int	 sl_add		__P((StringList *, char *));
 void	 sl_free	__P((StringList *, int));
 char	*sl_find	__P((StringList *, char *));
-#endif	/* ! HAVE_STRINGLIST_H */
 
 #if HAVE_TERMCAP_H
 # include <termcap.h>
@@ -142,10 +138,6 @@ void	 tputs		__P((const char *, int, int (*)(int)));
 # include "ftpvis.h"
 #endif
 
-#if ! HAVE_SIG_T
-typedef RETSIGTYPE (*sig_t) __P((int));
-#endif
-
 #if ! HAVE_IN_PORT_T
 typedef unsigned short in_port_t;
 #endif
@@ -218,7 +210,7 @@ char   *strdup		__P((const char *));
 char   *strerror	__P((int));
 #endif
 
-#if ! HAVE_STRPTIME
+#if ! HAVE_STRPTIME || ! HAVE_STRPTIME_D
 char   *strptime	__P((const char *, const char *, struct tm *));
 #endif
 
@@ -294,6 +286,3 @@ char   *strsep		__P((char **stringp, const char *delim));
 #define SECSPERHOUR	3600
 #define SECSPERDAY	86400
 #define TM_YEAR_BASE	1900
-
-			/* provided by src/utilc, used in libedit/sig.c */
-sig_t	xsignal_restart __P((int, void (func) __P((int)), int));
