@@ -1,4 +1,4 @@
-/*	$NetBSD: cmds.c,v 1.79 1999/11/11 02:53:02 lukem Exp $	*/
+/*	$NetBSD: cmds.c,v 1.82 2000/01/31 22:01:03 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1996-1999 The NetBSD Foundation, Inc.
@@ -109,6 +109,7 @@
 #include "lukemftp.h"
 
 #include "ftp_var.h"
+#include "version.h"
 
 struct	types {
 	char	*t_name;
@@ -433,7 +434,7 @@ mput(argc, argv)
 	char *argv[];
 {
 	int i;
-	sig_t oldintr;
+	sigfunc oldintr;
 	int ointer;
 	char *tp;
 
@@ -666,7 +667,7 @@ mget(argc, argv)
 	int argc;
 	char *argv[];
 {
-	sig_t oldintr;
+	sigfunc oldintr;
 	int ch, ointer;
 	char *cp, *tp, *tp2, tmpbuf[MAXPATHLEN];
 
@@ -1184,7 +1185,7 @@ mdelete(argc, argv)
 	int argc;
 	char *argv[];
 {
-	sig_t oldintr;
+	sigfunc oldintr;
 	int ointer;
 	char *cp;
 
@@ -1321,7 +1322,7 @@ mls(argc, argv)
 	int argc;
 	char *argv[];
 {
-	sig_t oldintr;
+	sigfunc oldintr;
 	int ointer, i;
 	int dolist;
 	char mode[1], *dest, *odest;
@@ -1379,7 +1380,7 @@ shell(argc, argv)
 	char *argv[];
 {
 	pid_t pid;
-	sig_t old1;
+	sigfunc old1;
 	char shellnam[MAXPATHLEN], *shell, *namep;
 	int wait_status;
 
@@ -1686,7 +1687,7 @@ idlecmd(argc, argv)
 {
 	int oldverbose = verbose;
 
-	if (argc != 2) {
+	if (argc < 1 || argc > 2) {
 		fprintf(ttyout, "usage: %s [seconds]\n", argv[0]);
 		code = -1;
 		return;
@@ -1806,7 +1807,7 @@ doproxy(argc, argv)
 {
 	struct cmd *c;
 	int cmdpos;
-	sig_t oldintr;
+	sigfunc oldintr;
 
 	if ((argc == 0 && argv != NULL) ||
 	    (argc == 1 && !another(&argc, &argv, "command"))) {
@@ -2234,7 +2235,7 @@ parserate(argc, argv, cmdlineopt)
 	int cmdlineopt;
 {
 	int dir, max, incr, showonly;
-	sig_t oldusr1, oldusr2;
+	sigfunc oldusr1, oldusr2;
 
 	if (argc > 4 || (argc < (cmdlineopt ? 3 : 2))) {
 usage:
