@@ -1,4 +1,4 @@
-/*	$NetBSD: fetch.c,v 1.96 1999/11/11 01:19:11 lukem Exp $	*/
+/*	$NetBSD: fetch.c,v 1.104 2000/01/31 22:01:04 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1997-1999 The NetBSD Foundation, Inc.
@@ -43,6 +43,7 @@
 
 #include "lukemftp.h"
 #include "ftp_var.h"
+#include "version.h"
 
 typedef enum {
 	UNKNOWN_URL_T=-1,
@@ -235,7 +236,7 @@ url_decode(url)
  * the number given, or ftpport if ftp://, or httpport if http://.
  *
  * If <host> is surrounded by `[' and ']', it's parsed as an
- * IPv6 address (as per draft-ietf-ipngwg-url-literal-01.txt).
+ * IPv6 address (as per RFC 2732).
  *
  * XXX: this is not totally RFC 1738 compliant; <path> will have the
  * leading `/' unless it's an ftp:// URL, as this makes things easier
@@ -338,7 +339,7 @@ cleanup_parse_url:
 #ifdef INET6
 			/*
 			 * Check if thost is an encoded IPv6 address, as per
-			 * draft-ietf-ipngwg-url-literal-01.txt:
+			 * RFC 2732:
 			 *	`[' ipv6-address ']'
 			 */
 	if (*thost == '[') {
@@ -349,7 +350,7 @@ cleanup_parse_url:
 			    thost, desc, origurl);
 			goto cleanup_parse_url;
 		}
-		len = ep - cp;		/* change `[xxx]' -> `xxx' */
+		len = ep - cp;		/* change `[xyz]' -> `xyz' */
 		memmove(thost, thost + 1, len);
 		thost[len] = '\0';
 		if (! isipv6addr(thost)) {
@@ -425,13 +426,14 @@ fetch_url(url, proxyenv, proxyauth, wwwauth)
 	char		*wwwauth;
 {
 #ifdef NI_NUMERICHOST
-	struct addrinfo		hints, *res = NULL;
+	struct addrinfo		hints, *res, *res0 = NULL;
 	int			error;
+	char			hbuf[NI_MAXHOST];
 #else
 	struct sockaddr_in	sin;
 	struct hostent		*hp = NULL;
 #endif
-	volatile sig_t		oldintr, oldintp;
+	volatile sigfunc	oldintr, oldintp;
 	volatile int		s;
 	struct stat		sb;
 	int			ischunked, isproxy, rval, hcode;
@@ -702,45 +704,48 @@ fetch_url(url, proxyenv, proxyauth, wwwauth)
 		hints.ai_family = AF_UNSPEC;
 		hints.ai_socktype = SOCK_STREAM;
 		hints.ai_protocol = 0;
-		error = getaddrinfo(host, port, &hints, &res);
+		error = getaddrinfo(host, port, &hints, &res0);
 		if (error) {
 			warnx(gai_strerror(error));
 			goto cleanup_fetch_url;
 		}
+		if (res0->ai_canonname)
+			host = res0->ai_canonname;
 
-		while (1) {
-			s = socket(res->ai_family,
-				   res->ai_socktype, res->ai_protocol);
+		s = -1;
+		for (res = res0; res; res = res->ai_next) {
+			if (getnameinfo(res->ai_addr, res->ai_addrlen,
+					hbuf, sizeof(hbuf), NULL, 0,
+					NI_NUMERICHOST) != 0)
+				strncpy(hbuf, "invalid", sizeof(hbuf));
+
+			if (verbose && res != res0)
+				fprintf(ttyout, "Trying %s...\n", hbuf);
+
+			s = socket(res->ai_family, res->ai_socktype,
+				res->ai_protocol);
 			if (s < 0) {
 				warn("Can't create socket");
-				goto cleanup_fetch_url;
+				continue;
 			}
 
 			if (xconnect(s, res->ai_addr, res->ai_addrlen) < 0) {
-				char hbuf[MAXHOSTNAMELEN];
-				getnameinfo(res->ai_addr, res->ai_addrlen,
-					hbuf, sizeof(hbuf), NULL, 0,
-					NI_NUMERICHOST);
 				warn("Connect to address `%s'", hbuf);
 				close(s);
-				res = res->ai_next;
-				if (res) {
-					getnameinfo(res->ai_addr,
-					    res->ai_addrlen, hbuf, sizeof(hbuf),
-					    NULL, 0, NI_NUMERICHOST);
-					if (verbose)
-						fprintf(ttyout,
-						    "Trying %s...\n", hbuf);
-					continue;
-				}
-				warn("Can't connect to %s", host);
-				goto cleanup_fetch_url;
+				s = -1;
+				continue;
 			}
-		
+
+			/* success */
 			break;
 		}
-#endif
+		freeaddrinfo(res0);
 
+		if (s < 0) {
+			warn("Can't connect to %s", host);
+			goto cleanup_fetch_url;
+		}
+#endif
 
 		fin = fdopen(s, "r+");
 		/*
@@ -1151,6 +1156,15 @@ fetch_url(url, proxyenv, proxyauth, wwwauth)
 				goto cleanup_fetch_url;
 			}
 			chunksize = strtol(xferbuf, &ep, 16);
+
+				/*
+				 * XXX:	Work around bug in Apache 1.3.9, which
+				 *	incorrectly puts a trailing space after
+				 *	the chunksize.
+				 */
+			if (*ep == ' ')
+				ep++;
+
 			if (strcmp(ep, "\r\n") != 0) {
 				warnx("Unexpected data following chunksize");
 				goto cleanup_fetch_url;
@@ -1175,10 +1189,11 @@ fetch_url(url, proxyenv, proxyauth, wwwauth)
 			if (rate_get)
 				(void)gettimeofday(&then, NULL);
 			bufrem = rate_get ? rate_get : bufsize;
+			if (ischunked)
+				bufrem = MIN(chunksize, bufrem);
 			while (bufrem > 0) {
 				len = fread(xferbuf, sizeof(char),
-				    ischunked ? MIN(chunksize, bufrem)
-					    : bufsize, fin);
+				    MIN(bufsize, bufrem), fin);
 				if (len <= 0)
 					goto chunkdone;
 				bytes += len;
@@ -1188,18 +1203,18 @@ fetch_url(url, proxyenv, proxyauth, wwwauth)
 					warn("Writing `%s'", savefile);
 					goto cleanup_fetch_url;
 				}
-			}
-			if (hash && !progress) {
-				while (bytes >= hashbytes) {
-					(void)putc('#', ttyout);
-					hashbytes += mark;
+				if (hash && !progress) {
+					while (bytes >= hashbytes) {
+						(void)putc('#', ttyout);
+						hashbytes += mark;
+					}
+					(void)fflush(ttyout);
+				}
+				if (ischunked) {
+					chunksize -= len;
+					if (chunksize <= 0)
+						break;
 				}
-				(void)fflush(ttyout);
-			}
-			if (ischunked) {
-				chunksize -= len;
-				if (chunksize <= 0)
-					goto chunkdone;
 			}
 			if (rate_get) {
 				while (1) {
@@ -1210,6 +1225,8 @@ fetch_url(url, proxyenv, proxyauth, wwwauth)
 					usleep(1000000 - td.tv_usec);
 				}
 			}
+			if (ischunked && chunksize <= 0)
+				break;
 		}
 					/* read CRLF after chunk*/
  chunkdone:
@@ -1269,10 +1286,6 @@ cleanup_fetch_url:
 		close(s);
 	if (closefunc != NULL && fout != NULL)
 		(*closefunc)(fout);
-#ifdef NI_NUMERICHOST
-	if (res != NULL)
-		freeaddrinfo(res);
-#endif
 	FREEPTR(savefile);
 	FREEPTR(user);
 	FREEPTR(pass);
@@ -1359,9 +1372,16 @@ fetch_ftp(url)
 			}
 			*cp = 0;
 		}
-	} else {			/* classic style `host:file' */
+	} else {			/* classic style `[user@]host:[file]' */
 		urltype = CLASSIC_URL_T;
 		host = xstrdup(url);
+		cp = strchr(host, '@');
+		if (cp != NULL) {
+			*cp = '\0';
+			user = host;
+			anonftp = 0;	/* disable anonftp */
+			host = xstrdup(cp + 1);
+		}
 		cp = strchr(host, ':');
 		if (cp != NULL) {
 			*cp = '\0';
@@ -1375,30 +1395,26 @@ fetch_ftp(url)
 	dir = path;
 	if (! EMPTYSTRING(dir)) {
 		/*
-		 * If we are dealing with classic `host:path' syntax,
-		 * then a path of the form `/file' (resulting from
-		 * input of the form `host:/file') means that we should
-		 * do "CWD /" before retrieving the file.  So we set
-		 * dir="/" and file="file".
+		 * If we are dealing with classic `[user@]host:[path]' syntax,
+		 * then a path of the form `/file' (resulting from input of the
+		 * form `host:/file') means that we should do "CWD /" before
+		 * retrieving the file.  So we set dir="/" and file="file".
 		 *
-		 * But if we are dealing with URLs like
-		 * `ftp://host/path' then a path of the form `/file'
-		 * (resulting from a URL of the form `ftp://host//file')
-		 * means that we should do `CWD ' (with an empty
-		 * argument) before retrieving the file.  So we set
+		 * But if we are dealing with URLs like `ftp://host/path' then
+		 * a path of the form `/file' (resulting from a URL of the form
+		 * `ftp://host//file') means that we should do `CWD ' (with an
+		 * empty argument) before retrieving the file.  So we set
 		 * dir="" and file="file".
 		 *
-		 * If the path does not contain / at all, we set
-		 * dir=NULL.  (We get a path without any slashes if
-		 * we are dealing with classic `host:file' or URL
-		 * `ftp://host/file'.)
+		 * If the path does not contain / at all, we set dir=NULL.
+		 * (We get a path without any slashes if we are dealing with
+		 * classic `[user@]host:[file]' or URL `ftp://host/file'.)
 		 *
-		 * In all other cases, we set dir to a string that does
-		 * not include the final '/' that separates the dir part
-		 * from the file part of the path.  (This will be the
-		 * empty string if and only if we are dealing with a
-		 * path of the form `/file' resulting from an URL of the
-		 * form `ftp://host//file'.)
+		 * In all other cases, we set dir to a string that does not
+		 * include the final '/' that separates the dir part from the
+		 * file part of the path.  (This will be the empty string if
+		 * and only if we are dealing with a path of the form `/file'
+		 * resulting from an URL of the form `ftp://host//file'.)
 		 */
 		cp = strrchr(dir, '/');
 		if (cp == dir && urltype == CLASSIC_URL_T) {
@@ -1477,8 +1493,8 @@ fetch_ftp(url)
 		char *nextpart;
 
 		/*
-		 * If we are dealing with a classic `host:path' (urltype
-		 * is CLASSIC_URL_T) then we have a raw directory
+		 * If we are dealing with a classic `[user@]host:[path]'
+		 * (urltype is CLASSIC_URL_T) then we have a raw directory
 		 * name (not encoded in any way) and we can change
 		 * directories in one step.
 		 *
