@@ -1,4 +1,4 @@
-/*	$NetBSD: ftp.c,v 1.87 1999/11/11 01:19:12 lukem Exp $	*/
+/*	$NetBSD: ftp.c,v 1.90 2000/01/31 22:01:04 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1996-1999 The NetBSD Foundation, Inc.
@@ -313,14 +313,12 @@ hookup(host, port)
 	if (myctladdr.su_len == 0)
 		myctladdr.su_len = len;
 
-#if defined(IPPROTO_IP) && defined(IP_TOS)
 	if (family == AF_INET) {
 		int tos = IPTOS_LOWDELAY;
 		if (setsockopt(s, IPPROTO_IP, IP_TOS, (char *)&tos,
 			       sizeof(int)) < 0)
 			warn("setsockopt TOS (ignored)");
 	}
-#endif
 	cin = fdopen(s, "r");
 	cout = fdopen(s, "w");
 	if (cin == NULL || cout == NULL) {
@@ -342,7 +340,6 @@ hookup(host, port)
 		code = -1;
 		goto bad;
 	}
-#ifdef SO_OOBINLINE
 	{
 	int on = 1;
 
@@ -351,7 +348,6 @@ hookup(host, port)
 			warn("setsockopt");
 		}
 	}
-#endif /* SO_OOBINLINE */
 
 	return (hostname);
 bad:
@@ -401,7 +397,7 @@ command(va_alist)
 {
 	va_list ap;
 	int r;
-	sig_t oldsigint;
+	sigfunc oldsigint;
 #ifndef __STDC__
 	const char *fmt;
 #endif
@@ -459,7 +455,7 @@ getreply(expecteof)
 	int c, n, line;
 	int dig;
 	int originalcode = 0, continuation = 0;
-	sig_t oldsigint, oldsigalrm;
+	sigfunc oldsigint, oldsigalrm;
 	int pflag = 0;
 	char *cp, *pt = pasv;
 
@@ -702,7 +698,7 @@ sendrequest(cmd, local, remote, printnames)
 	int c, d;
 	FILE *fin, *dout;
 	int (*closefunc) __P((FILE *));
-	sig_t oldintr, oldintp;
+	sigfunc oldintr, oldintp;
 	volatile off_t hashbytes;
 	char *lmode, *bufp;
 	static size_t bufsize;
@@ -835,20 +831,50 @@ sendrequest(cmd, local, remote, printnames)
 
 	case TYPE_I:
 	case TYPE_L:
-		while (1) {
-			struct timeval then, now, td;
-			off_t bufrem;
+		if (rate_put) {		/* rate limited */
+			while (1) {
+				struct timeval then, now, td;
+				off_t bufrem;
 
-			if (rate_put)
 				(void)gettimeofday(&then, NULL);
-			errno = c = d = 0;
-			bufrem = rate_put ? rate_put : bufsize;
-			while (bufrem > 0) {
-				if ((c = read(fileno(fin), buf,
-				    MIN(bufsize, bufrem))) <= 0)
+				errno = c = d = 0;
+				bufrem = rate_put;
+				while (bufrem > 0) {
+					if ((c = read(fileno(fin), buf,
+					    MIN(bufsize, bufrem))) <= 0)
+						goto senddone;
+					bytes += c;
+					bufrem -= c;
+					for (bufp = buf; c > 0;
+					    c -= d, bufp += d)
+						if ((d = write(fileno(dout),
+						    bufp, c)) <= 0)
+							break;
+					if (d < 0)
+						goto senddone;
+					if (hash &&
+					    (!progress || filesize < 0) ) {
+						while (bytes >= hashbytes) {
+							(void)putc('#', ttyout);
+							hashbytes += mark;
+						}
+						(void)fflush(ttyout);
+					}
+				}
+				while (1) {
+					(void)gettimeofday(&now, NULL);
+					timersub(&now, &then, &td);
+					if (td.tv_sec > 0)
+						break;
+					usleep(1000000 - td.tv_usec);
+				}
+			}
+		} else {		/* simpler/faster no rate limit */
+			while (1) {
+				errno = c = d = 0;
+				if ((c = read(fileno(fin), buf, bufsize)) <= 0)
 					goto senddone;
 				bytes += c;
-				bufrem -= c;
 				for (bufp = buf; c > 0; c -= d, bufp += d)
 					if ((d = write(fileno(dout), bufp, c))
 					    <= 0)
@@ -863,15 +889,6 @@ sendrequest(cmd, local, remote, printnames)
 					(void)fflush(ttyout);
 				}
 			}
-			if (rate_put) {
-				while (1) {
-					(void)gettimeofday(&now, NULL);
-					timersub(&now, &then, &td);
-					if (td.tv_sec > 0)
-						break;
-					usleep(1000000 - td.tv_usec);
-				}
-			}
 		}
  senddone:
 		if (hash && (!progress || filesize < 0) && bytes > 0) {
@@ -983,7 +1000,7 @@ recvrequest(cmd, local, remote, lmode, printnames, ignorespecial)
 {
 	FILE *fout, *din;
 	int (*closefunc) __P((FILE *));
-	sig_t oldintr, oldintp;
+	sigfunc oldintr, oldintp;
 	int c, d;
 	volatile int is_retr, tcrflag, bare_lfs;
 	static size_t bufsize;
@@ -1096,7 +1113,7 @@ recvrequest(cmd, local, remote, lmode, printnames, ignorespecial)
 	    command("REST %ld", (long) restart_point) != CONTINUE)
 #endif
 		goto cleanuprecv;
-	if (remote) {
+	if (! EMPTYSTRING(remote)) {
 		if (command("%s %s", cmd, remote) != PRELIM)
 			goto cleanuprecv;
 	} else {
@@ -1151,20 +1168,46 @@ recvrequest(cmd, local, remote, lmode, printnames, ignorespecial)
 			warn("local: %s", local);
 			goto cleanuprecv;
 		}
-		while (1) {
-			struct timeval then, now, td;
-			off_t bufrem;
+		if (rate_get) {		/* rate limiting */
+			while (1) {
+				struct timeval then, now, td;
+				off_t bufrem;
 
-			if (rate_get)
 				(void)gettimeofday(&then, NULL);
-			errno = c = d = 0;
-			bufrem = rate_get ? rate_get : bufsize;
-			while (bufrem > 0) {
-				if ((c = read(fileno(din), buf,
-				    MIN(bufsize, bufrem))) <= 0)
+				errno = c = d = 0;
+				for (bufrem = rate_get; bufrem > 0; ) {
+					if ((c = read(fileno(din), buf,
+					    MIN(bufsize, bufrem))) <= 0)
+						goto recvdone;
+					bytes += c;
+					bufrem -=c;
+					if ((d = write(fileno(fout), buf, c))
+					    != c)
+						goto recvdone;
+					if (hash &&
+					    (!progress || filesize < 0)) {
+						while (bytes >= hashbytes) {
+							(void)putc('#', ttyout);
+							hashbytes += mark;
+						}
+						(void)fflush(ttyout);
+					}
+				}
+					/* sleep until time is up */
+				while (1) {
+					(void)gettimeofday(&now, NULL);
+					timersub(&now, &then, &td);
+					if (td.tv_sec > 0)
+						break;
+					usleep(1000000 - td.tv_usec);
+				}
+			}
+		} else {		/* faster code (no limiting) */
+			while (1) {
+				errno = c = d = 0;
+				if ((c = read(fileno(din), buf, bufsize)) <= 0)
 					goto recvdone;
 				bytes += c;
-				bufrem -=c;
 				if ((d = write(fileno(fout), buf, c)) != c)
 					goto recvdone;
 				if (hash && (!progress || filesize < 0)) {
@@ -1175,15 +1218,6 @@ recvrequest(cmd, local, remote, lmode, printnames, ignorespecial)
 					(void)fflush(ttyout);
 				}
 			}
-			if (rate_get) {
-				while (1) {
-					(void)gettimeofday(&now, NULL);
-					timersub(&now, &then, &td);
-					if (td.tv_sec > 0)
-						break;
-					usleep(1000000 - td.tv_usec);
-				}
-			}
 		}
  recvdone:
 		if (hash && (!progress || filesize < 0) && bytes > 0) {
@@ -1596,14 +1630,12 @@ reinit:
 			warn("connect");
 			goto bad;
 		}
-#if defined(IPPROTO_IP) && defined(IP_TOS)
 		if (data_addr.su_family == AF_INET) {
 			on = IPTOS_THROUGHPUT;
 			if (setsockopt(data, IPPROTO_IP, IP_TOS, (char *)&on,
 				       sizeof(int)) < 0)
 				warn("setsockopt TOS (ignored)");
 		}
-#endif
 		return (0);
 	}
 
@@ -1724,14 +1756,12 @@ noport:
 	}
 	if (tmpno)
 		sendport = 1;
-#if defined(IPPROTO_IP) && defined(IP_TOS)
 	if (data_addr.su_family == AF_INET) {
 		on = IPTOS_THROUGHPUT;
 		if (setsockopt(data, IPPROTO_IP, IP_TOS, (char *)&on,
 			       sizeof(int)) < 0)
 			warn("setsockopt TOS (ignored)");
 	}
-#endif
 	return (0);
 bad:
 	(void)close(data), data = -1;
@@ -1758,7 +1788,6 @@ dataconn(lmode)
 	}
 	(void)close(data);
 	data = s;
-#if defined(IPPROTO_IP) && defined(IP_TOS)
 	if (from.su_family == AF_INET) {
 		int tos = IPTOS_THROUGHPUT;
 		if (setsockopt(s, IPPROTO_IP, IP_TOS, (char *)&tos,
@@ -1766,7 +1795,6 @@ dataconn(lmode)
 			warn("setsockopt TOS (ignored)");
 		}
 	}
-#endif
 	return (fdopen(data, lmode));
 }
 
@@ -1785,7 +1813,7 @@ void
 pswitch(flag)
 	int flag;
 {
-	sig_t oldintr;
+	sigfunc oldintr;
 	static struct comvars {
 		int connect;
 		char name[MAXHOSTNAMELEN];
@@ -1887,7 +1915,7 @@ void
 proxtrans(cmd, local, remote)
 	const char *cmd, *local, *remote;
 {
-	sig_t oldintr;
+	sigfunc oldintr;
 	int prox_type, nfnd;
 	volatile int secndflag;
 	char *cmd2;
