@@ -1,4 +1,4 @@
-/*	$NetBSD: ftp_var.h,v 1.48 1999/11/12 02:50:38 lukem Exp $	*/
+/*	$NetBSD: ftp_var.h,v 1.52 2000/01/31 22:01:05 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1996-1999 The NetBSD Foundation, Inc.
@@ -104,13 +104,6 @@
  * FTP global variables.
  */
 
-#ifndef FTP_PRODUCT
-#define	FTP_PRODUCT	"NetBSD-ftp"
-#endif
-#ifndef FTP_VERSION
-#define	FTP_VERSION	"19991111"
-#endif
-
 #ifdef SMALL
 #undef	NO_EDITCOMPLETE
 #define	NO_EDITCOMPLETE
@@ -122,6 +115,8 @@
 #include <histedit.h>
 #endif /* !NO_EDITCOMPLETE */
 
+typedef void (*sigfunc) __P((int));
+
 #include "extern.h"
 
 
