@@ -1,4 +1,4 @@
-/*	$NetBSD: main.c,v 1.67 1999/11/12 02:50:38 lukem Exp $	*/
+/*	$NetBSD: main.c,v 1.69 1999/11/28 06:32:05 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1996-1999 The NetBSD Foundation, Inc.
@@ -195,7 +195,7 @@ main(argc, argv)
 	if (sndbuf_size <= 0)
 		sndbuf_size = 8192;
 
-	marg_sl = sl_init();
+	marg_sl = xsl_init();
 	if ((tmpdir = getenv("TMPDIR")) == NULL)
 		tmpdir = _PATH_TMP;
 
@@ -430,19 +430,36 @@ main(argc, argv)
 			if (rval >= 0)		/* -1 == connected and cd-ed */
 				exit(rval);
 		} else {
-			char *xargv[5];
+			char *xargv[4], *user, *host;
 
 			if (sigsetjmp(toplevel, 1))
 				exit(0);
 			(void)xsignal(SIGINT, intr);
 			(void)xsignal(SIGPIPE, lostpeer);
+			user = NULL;
+			host = argv[0];
+			cp = strchr(host, '@');
+			if (cp) {
+				*cp = '\0';
+				user = host;
+				host = cp + 1;
+			}
 			xargv[0] = __progname;
-			xargv[1] = argv[0];
+			xargv[1] = host;
 			xargv[2] = argv[1];
-			xargv[3] = argv[2];
-			xargv[4] = NULL;
+			xargv[3] = NULL;
 			do {
+				int oautologin;
+
+				oautologin = autologin;
+				if (user != NULL) {
+					anonftp = 0;
+					autologin = 0;
+				}
 				setpeer(argc+1, xargv);
+				autologin = oautologin;
+				if (connected == 1 && user != NULL)
+					(void)ftp_login(host, user, NULL);
 				if (!retry_connect)
 					break;
 				if (!connected) {
@@ -659,7 +676,7 @@ makeargv()
 	marg_sl->sl_cur = 0;		/* reset to start of marg_sl */
 	for (margc = 0; ; margc++) {
 		argp = slurpstring();
-		sl_add(marg_sl, argp);
+		xsl_add(marg_sl, argp);
 		if (argp == NULL)
 			break;
 	}
@@ -841,13 +858,13 @@ help(argc, argv)
 	if (argc == 1) {
 		StringList *buf;
 
-		buf = sl_init();
+		buf = xsl_init();
 		fprintf(ttyout,
 		    "%sommands may be abbreviated.  Commands are:\n\n",
 		    proxy ? "Proxy c" : "C");
 		for (c = cmdtab; (p = c->c_name) != NULL; c++)
 			if (!proxy || c->c_proxy)
-				sl_add(buf, p);
+				xsl_add(buf, p);
 		list_vertical(buf);
 		sl_free(buf, 0);
 		return;
@@ -927,8 +944,9 @@ void
 usage()
 {
 	(void)fprintf(stderr,
-"usage: %s [-AadefginpRtvV] [-r retry] [-o outfile] [-P port] [-T dir,max[,inc]\n"
-"       [host [port]] [file:///file] [ftp://[user[:pass]@]host[:port]/path[/]]\n"
-"       [http://[user[:pass]@]host[:port]/path] [host:path[/]]\n", __progname);
+"usage: %s [-AadefginpRtvV] [-o outfile] [-P port] [-r retry] [-T dir,max[,inc]\n"
+"       [[user@]host [port]] [host:path[/]] [file:///file]\n"
+"       [ftp://[user[:pass]@]host[:port]/path[/]]\n"
+"       [http://[user[:pass]@]host[:port]/path] [...]\n", __progname);
 	exit(1);
 }
