@@ -1,4 +1,4 @@
-/*	$NetBSD: util.c,v 1.82 1999/11/12 02:50:38 lukem Exp $	*/
+/*	$NetBSD: util.c,v 1.91 2000/01/31 22:01:05 lukem Exp $	*/
 
 /*-
  * Copyright (c) 1997-1999 The NetBSD Foundation, Inc.
@@ -199,7 +199,6 @@ setpeer(argc, argv)
 "Remember to set tenex mode when transferring binary files from this machine.\n",
 				    ttyout);
 		}
-		updateremotepwd();
 		verbose = overbose;
 	}
 }
@@ -229,6 +228,7 @@ cleanuppeer()
 	epsv4bad = 0;
 	if (username)
 		free(username);
+	username = NULL;
 	if (!proxy)
 		macnum = 0;
 }
@@ -303,6 +303,12 @@ ftp_login(host, user, pass)
 	acct = NULL;
 	aflag = rval = freeuser = freepass = freeacct = 0;
 
+	if (debug)
+		fprintf(ttyout, "ftp_login: user `%s' pass `%s' host `%s'\n",
+		    user ? user : "<null>", pass ? pass : "<null>",
+		    host ? host : "<null>");
+
+
 	/*
 	 * Set up arguments for an anonymous FTP session, if necessary.
 	 */
@@ -397,6 +403,8 @@ ftp_login(host, user, pass)
 			break;
 		}
 	}
+	updateremotepwd();
+
 cleanup_ftp_login:
 	if (user != NULL && freeuser)
 		free((char *)user);
@@ -612,25 +620,67 @@ remotemodtime(file, noisy)
 		verbose = -1;
 	if (command("MDTM %s", file) == COMPLETE) {
 		struct tm timebuf;
+		char *timestr, *frac;
 		int yy, mo, day, hour, min, sec;
-		sscanf(reply_string, "%*s %04d%02d%02d%02d%02d%02d", &yy, &mo,
-			&day, &hour, &min, &sec);
+
+		/*
+		 * time-val = 14DIGIT [ "." 1*DIGIT ]
+		 *		YYYYMMDDHHMMSS[.sss]
+		 * mdtm-response = "213" SP time-val CRLF / error-response
+		 */
+		timestr = reply_string + 4;
+
+					/*
+					 * parse fraction.
+					 * XXX: ignored for now
+					 */
+		frac = strchr(timestr, '\r');
+		if (frac != NULL)
+			*frac = '\0';
+		frac = strchr(timestr, '.');
+		if (frac != NULL)
+			*frac++ = '\0';
+		if (strlen(timestr) == 15 && strncmp(timestr, "191", 3) == 0) {
+			/*
+			 * XXX:	Workaround for lame ftpd's that return
+			 *	`19100' instead of `2000'
+			 */
+			fprintf(ttyout,
+	    "Y2K warning! Incorrect time-val `%s' received from server.\n",
+			    timestr);
+			timestr++;
+			timestr[0] = '2';
+			timestr[1] = '0';
+			fprintf(ttyout, "Converted to `%s'\n", timestr);
+		}
+		if (strlen(timestr) != 14 ||
+		    sscanf(timestr, "%04d%02d%02d%02d%02d%02d",
+			&yy, &mo, &day, &hour, &min, &sec) != 6) {
+ bad_parse_time:
+			fprintf(ttyout, "Can't parse time `%s'.\n", timestr);
+			goto cleanup_parse_time;
+		}
 		memset(&timebuf, 0, sizeof(timebuf));
 		timebuf.tm_sec = sec;
 		timebuf.tm_min = min;
 		timebuf.tm_hour = hour;
 		timebuf.tm_mday = day;
 		timebuf.tm_mon = mo - 1;
-		timebuf.tm_year = yy - TM_YEAR_BASE;
+		timebuf.tm_year = yy - TM_YEAR_BASE; 
 		timebuf.tm_isdst = -1;
 		rtime = timegm(&timebuf);
-		if (rtime == -1 && (noisy || debug != 0))
-			fprintf(ttyout, "Can't convert %s to a time.\n",
-			    reply_string);
+		if (rtime == -1) {
+			if (noisy || debug != 0)
+				goto bad_parse_time;
+			else
+				goto cleanup_parse_time;
+		} else if (debug)
+			fprintf(ttyout, "parsed date as: %s", ctime(&rtime));
 	} else if (noisy && debug == 0) {
 		fputs(reply_string, ttyout);
 		putc('\n', ttyout);
 	}
+ cleanup_parse_time:
 	verbose = overbose;
 	if (rtime == -1)
 		code = ocode;
@@ -1348,7 +1398,7 @@ isipv6addr(addr)
 	if (debug)
 		fprintf(ttyout, "isipv6addr: got %d for %s\n", rv, addr);
 #endif
-	return rv;
+	return (rv == 1) ? 1 : 0;
 }
 
 
@@ -1393,6 +1443,33 @@ xmalloc(size)
 	return (p);
 }
 
+/*
+ * sl_init() with inbuilt error checking
+ */
+StringList *
+xsl_init()
+{
+	StringList *p;
+
+	p = sl_init();
+	if (p == NULL)
+		err(1, "Unable to allocate memory for stringlist");
+	return (p);
+}
+
+/*
+ * sl_add() with inbuilt error checking
+ */
+void
+xsl_add(sl, i)
+	StringList	*sl;
+	char		*i;
+{
+
+	if (sl_add(sl, i) == -1)
+		err(1, "Unable to add `%s' to stringlist", i);
+}
+
 /*
  * strdup() with inbuilt error checking
  */
@@ -1414,10 +1491,10 @@ xstrdup(str)
  * Install a POSIX signal handler, allowing the invoker to set whether
  * the signal should be restartable or not
  */
-sig_t
+sigfunc
 xsignal_restart(sig, func, restartable)
 	int sig;
-	void (*func) __P((int));
+	sigfunc func;
 	int restartable;
 {
 #ifdef ultrix	/* XXX: this is lame - how do we test sigvec vs. sigaction? */
@@ -1451,10 +1528,10 @@ xsignal_restart(sig, func, restartable)
  * Install a signal handler with the `restartable' flag set dependent upon
  * which signal is being set. (This is a wrapper to xsignal_restart())
  */
-sig_t
+sigfunc
 xsignal(sig, func)
 	int sig;
-	void (*func) __P((int));
+	sigfunc func;
 {
 	int restartable;
 
